MODULE bdydta
   !!==============================================================================
   !!                            ***  MODULE bdydta  ***
   !! Open boundary data : read the data for the unstructured open boundaries.
   !!==============================================================================
#if defined key_bdy || defined key_bdy_tides
   !!------------------------------------------------------------------------------
   !!   'key_bdy'         :                   Unstructured Open Boundary Conditions
   !!------------------------------------------------------------------------------
   !!   bdy_dta           : read u, v, t, s data along each open boundary
   !!------------------------------------------------------------------------------
   !! * Modules used
   USE oce             ! ocean dynamics and tracers
   USE dom_oce         ! ocean space and time domain
   USE daymod          ! calendar
   USE phycst          ! physical constants
   USE bdy_oce         ! ocean open boundary conditions
   USE bdytides        ! tidal forcing at boundaries
   USE iom
   USE ioipsl
   USE in_out_manager  ! I/O logical units

   IMPLICIT NONE
   PRIVATE

   !! * Accessibility
   PUBLIC bdy_dta                         ! routines called by step.F90
   PUBLIC bdy_dta_bt 
   
   !! * local modules variables
   INTEGER ::   &
      bdynumt,         &  ! logical unit for T-points data file
      bdynumu,         &  ! logical unit for U-points data file
      bdynumv,         &  ! logical unit for V-points data file
      kbdy,            &  ! Exact number of time dumps in data files
      nbdy1,           &  ! Time record in bdy data file BEFORE the model current time
      nbdy2               ! Time record in bdy data file AFTER the model current time

   !! * local modules variables for barotropic variables
   INTEGER ::   &
      bdynumt_bt,         &  ! logical unit for T-points data file
      bdynumu_bt,         &  ! logical unit for U-points data file
      bdynumv_bt,         &  ! logical unit for V-points data file
      kbdy_bt,            &  ! Exact number of time dumps in data files
      nbdy1_bt,           &  ! Time record in bdy data file BEFORE the model current time
      nbdy2_bt               ! Time record in bdy data file AFTER the model current time

   INTEGER, DIMENSION (jpbtime) ::   & 
      istep, istep_bt        ! time array in seconds in each data file

   REAL(wp) ::  &
      offset              ! time offset between time origin in file and start time of model run

   REAL(wp), DIMENSION(jpbdim,jpk,2) ::   &
      tbdydta, sbdydta,	& !: Arrays used for time interpolation of bdy data   
      ubdydta, vbdydta

   REAL(wp), DIMENSION(jpbdim,2) ::   &
      ubtbdydta, vbtbdydta,	& !: Arrays used for time interpolation of bdy data   
      sshbdydta                   !: bdy data of ssh

   !!---------------------------------------------------------------------------------
   !!   OPA 9.0 , LODYC-IPSL  (2003)
   !!---------------------------------------------------------------------------------

CONTAINS

   SUBROUTINE bdy_dta( kt )
      !!---------------------------------------------------------------------------
      !!                      ***  SUBROUTINE bdy_dta  ***
      !!                    
      !! ** Purpose :   Read unstructured boundary data
      !!
      !! ** Method  :   
      !!
      !! History :  OPA 9.0 ! 05-01 (J. Chanut, A. Sellar) Original
      !!                 "  ! 07-01 (D. Storkey) Update to use IOM module.
      !!---------------------------------------------------------------------------
      !! * Arguments
      INTEGER, INTENT( in ) ::   kt          ! ocean time-step index
                                             ! (for timesplitting option, otherwise zero)

      !! * Local declarations
      LOGICAL ::   lect                      ! flag for reading
      INTEGER ::   jt, jb, jk, jgrd          ! dummy loop indices
      INTEGER ::   idvar                     ! netcdf var ID
      INTEGER ::   iman, i15, imois          ! Time variables for monthly clim forcing
      INTEGER ::   kbdyt, kbdyu, kbdyv
      INTEGER ::   itimer, totime
      INTEGER ::   ipi, ipj, ipk, inum       ! temporary integers (NetCDF read)
      INTEGER ::   iyear0, imonth0, iday0
      INTEGER ::   ihours0, iminutes0, isec0
      INTEGER ::   kyear, kmonth, kday, ksecs
      INTEGER, DIMENSION(jpbtime) ::   istept, istepu, istepv   ! time arrays from data files
      REAL(wp) ::   dayfrac, zxy, offsett
      REAL(wp) ::   offsetu, offsetv
      REAL(wp) ::   dayjul0, zdayjulini
      REAL(wp), DIMENSION(jpbtime)      ::   istepr             ! REAL time array from data files
      REAL(wp), DIMENSION(jpbdta,1,jpk) ::   pdta3              ! temporary array for data fields
      CHARACTER(LEN=80), DIMENSION(3)   ::   bdyfile
      CHARACTER(LEN=70 )                ::   clunits            ! units attribute of time coordinate

      !!---------------------------------------------------------------------------

      ! -------------------- !
      !    Initialization    !
      ! -------------------- !

      lect   = .false.           ! If true, read a time record

      ! Some time variables for monthly climatological forcing:
      ! *******************************************************
 
      iman  = INT( raamo ) ! Number of months in a year

      i15 = INT( 2*FLOAT( nday ) / ( FLOAT( nobis(nmonth) ) + 0.5 ) )
      ! i15=0 if the current day is in the first half of the month, else i15=1

      imois = nmonth + i15 - 1            ! imois is the first month record
      IF( imois == 0 ) imois = iman

      ! Time variable for non-climatological forcing:
      ! *********************************************

      itimer = (kt-nit000+1)*rdt      ! current time in seconds for interpolation 

      ! -------------------- !
      ! 1.   First call only !
      ! -------------------- !

      IF ( kt == nit000 ) THEN

        istep(:) = 0

        nbdy1=0
        nbdy2=0

      ! 1.1  Get time information from bdy data file
      ! ********************************************

        IF(lwp) WRITE(numout,*)
        IF(lwp) WRITE(numout,*)    'bdy_dta :Initialize unstructured boundary data'
        IF(lwp) WRITE(numout,*)    '~~~~~~~' 

        IF (nbdy_dta == 0) THEN
          IF(lwp) WRITE(numout,*)  'Bdy data are taken from initial conditions'

        ELSEIF (nbdy_dta == 1) THEN
          IF(lwp) WRITE(numout,*)  'Bdy data are read in netcdf files'

          dayfrac = adatrj-float(itimer)/86400. ! day fraction at time step kt-1
          dayfrac = dayfrac - INT(dayfrac)      !
          totime = (nitend-nit000+1)*rdt ! Total time of the run to verify that all the
                                           ! necessary time dumps in file are included

          bdyfile(1) = filbdy_data_T
          bdyfile(2) = filbdy_data_U
          bdyfile(3) = filbdy_data_V

          DO jgrd = 1,3

            CALL iom_open( bdyfile(jgrd), inum )
            CALL iom_gettime( inum, istepr, kntime=kbdy, cdunits=clunits ) 
            CALL iom_close( inum )

            ! Calculate time offset 
            READ(clunits,7000) iyear0, imonth0, iday0, ihours0, iminutes0, isec0
            ! Convert time origin in file to julian days 
            isec0 = isec0 + ihours0*60.*60. + iminutes0*60.
            CALL ymds2ju(iyear0, imonth0, iday0, real(isec0), dayjul0)
            ! Compute model initialization time 
            kyear  = ndastp / 10000
            kmonth = ( ndastp - kyear * 10000 ) / 100
            kday   = ndastp - kyear * 10000 - kmonth * 100
            ksecs  = dayfrac * 86400
            CALL ymds2ju(kyear, kmonth, kday, real(ksecs) , zdayjulini)
            ! offset from initialization date:
            offset = (dayjul0-zdayjulini)*86400
            !

7000 FORMAT('seconds since ', I4.4,'-',I2.2,'-',I2.2,' ',I2.2,':',I2.2,':',I2.2)


            !! TO BE DONE... Check consistency between calendar from file 
            !! (available optionally from iom_gettime) and calendar in model 
            !! when calendar in model available outside of IOIPSL.

            IF(lwp) WRITE(numout,*) 'kdby (number of times): ',kbdy
            IF(lwp) WRITE(numout,*) 'offset: ',offset
            IF(lwp) WRITE(numout,*) 'totime: ',totime
            IF(lwp) WRITE(numout,*) 'istepr: ',istepr

            ! Check that there are not too many times in the file. 
            IF (kbdy > jpbtime) THEN
              IF(lwp) WRITE(numout,*)
              IF(lwp) WRITE(numout,*) ' E R R O R : Number of time dumps in files exceed jpbtime parameter'
              IF(lwp) WRITE(numout,*) ' ==========  jpbtime= ',jpbtime,' kbdy= ', kbdy             
              IF(lwp) WRITE(numout,*) ' Check file: ', bdyfile(jgrd)
              nstop = nstop + 1
            ENDIF

            ! Check that time array increases:
        
            jt=1
            DO WHILE ((istepr(jt+1).GT.istepr(jt)).AND.(jt.LE.(kbdy-1)))
              jt=jt+1
            END DO

            IF ((jt.NE.kbdy).AND.(kbdy.GT.1)) THEN
              IF(lwp) WRITE(numout,*)
              IF(lwp) WRITE(numout,*) ' E R R O R : Time array in unstructured boundary data files'
              IF(lwp) WRITE(numout,*) ' ==========  does not continuously increase. Check file: '         
              IF(lwp) WRITE(numout,*) bdyfile(jgrd)
              IF(lwp) WRITE(numout,*)
              nstop = nstop + 1
            ENDIF

            ! Check that times in file span model run time:
 
            IF (istepr(1)+offset > 0) THEN
              IF(lwp) WRITE(numout,*)
              IF(lwp) WRITE(numout,*) ' E R R O R : First time dump in bdy file is after model initial time'
              IF(lwp) WRITE(numout,*) ' ==========  Check file: ',bdyfile(jgrd)
              IF(lwp) WRITE(numout,*)
              nstop = nstop + 1
            END IF

            IF (istepr(kbdy)+offset < totime) THEN
              IF(lwp) WRITE(numout,*)
              IF(lwp) WRITE(numout,*) ' E R R O R : Last time dump in bdy file is before model final time'
              IF(lwp) WRITE(numout,*) ' ==========  Check file: ',bdyfile(jgrd) 
              IF(lwp) WRITE(numout,*)
              nstop = nstop + 1
            END IF

            IF ( jgrd .EQ. 1) THEN
              kbdyt = kbdy
              offsett = offset
              istept(:) = INT( istepr(:) + offset )
            ELSE IF (jgrd .EQ. 2) THEN
              kbdyu = kbdy
              offsetu = offset
              istepu(:) = INT( istepr(:) + offset )
            ELSE IF (jgrd .EQ. 3) THEN
              kbdyv = kbdy
              offsetv = offset
              istepv(:) = INT( istepr(:) + offset )
            ENDIF

          ENDDO

      ! Verify time consistency between files  

          IF (kbdyu.NE.kbdyt .OR. kbdyv.NE.kbdyt) THEN
            IF(lwp) WRITE(numout,*) 'Bdy data files must have the same number of time dumps'
            IF(lwp) WRITE(numout,*) 'Multiple time frequencies not implemented yet'
            nstop = nstop + 1
          ENDIF
          kbdy = kbdyt

          IF (offsetu.NE.offsett .OR. offsetv.NE.offsett) THEN
            IF(lwp) WRITE(numout,*) 'Bdy data files must have the same time origin'
            IF(lwp) WRITE(numout,*) 'Multiple time frequencies not implemented yet'
            nstop = nstop + 1
          ENDIF
          offset = offsett

      !! Check that times are the same in the three files... HERE.
          istep(:) = istept(:)

      ! Check number of time dumps:              
          IF ((kbdy.EQ.1).AND.(.NOT.ln_bdy_clim)) THEN
            IF(lwp) WRITE(numout,*)
            IF(lwp) WRITE(numout,*) ' E R R O R : There is only one time dump in data files'
            IF(lwp) WRITE(numout,*) ' ==========  Choose ln_bdy_clim=.true. in namelist for constant bdy forcing.'            
            IF(lwp) WRITE(numout,*)
            nstop = nstop + 1
          ENDIF

          IF (ln_bdy_clim) THEN
            IF ((kbdy.NE.1).AND.(kbdy.NE.12)) THEN
              IF(lwp) WRITE(numout,*)
              IF(lwp) WRITE(numout,*) ' E R R O R : For climatological boundary forcing (ln_bdy_clim=.true.),'
              IF(lwp) WRITE(numout,*) ' ==========  bdy data files must contain 1 or 12 time dumps.'         
              IF(lwp) WRITE(numout,*)
              nstop = nstop + 1
            ELSEIF (kbdy.EQ.1 ) THEN
              IF(lwp) WRITE(numout,*)
              IF(lwp) WRITE(numout,*) 'We assume constant boundary forcing from bdy data files'
              IF(lwp) WRITE(numout,*)             
            ELSEIF (kbdy.EQ.12) THEN
              IF(lwp) WRITE(numout,*)
              IF(lwp) WRITE(numout,*) 'We assume monthly (and cyclic) boundary forcing from bdy data files'
              IF(lwp) WRITE(numout,*) 
            ENDIF
          ENDIF

      ! Find index of first record to read (before first model time). 

          jt=1
          DO WHILE ( ((istep(jt+1)) <= 0 ).AND.(jt.LE.(kbdy-1)))
            jt=jt+1
          END DO
          nbdy1 = jt

          WRITE(numout,*) 'Time offset is ',offset
          WRITE(numout,*) 'First record to read is ',nbdy1

        ENDIF ! endif (nbdy_dta == 1)

      ! 1.2  Read first record in file if necessary (ie if nbdy_dta == 1)
      ! *****************************************************************

        IF ( nbdy_dta == 0) THEN
          ! boundary data arrays are filled with initial conditions
          DO jk = 1, jpkm1 

#if ! defined key_barotropic
            jgrd = 1            ! T-points data 
            DO jb = 1, nblen(jgrd)              
              tbdy(jb,jk) = tn(nbi(jb,jgrd), nbj(jb,jgrd), jk)
              sbdy(jb,jk) = sn(nbi(jb,jgrd), nbj(jb,jgrd), jk)
            END DO
#endif

            jgrd = 2            ! U-points data 
            DO jb = 1, nblen(jgrd)              
              ubdy(jb,jk) = un(nbi(jb,jgrd), nbj(jb,jgrd), jk)
            END DO

            jgrd = 3            ! V-points data 
            DO jb = 1, nblen(jgrd)              
              vbdy(jb,jk) = vn(nbi(jb,jgrd), nbj(jb,jgrd), jk)
            END DO
          END DO 

        ELSEIF (nbdy_dta == 1) THEN
 
        ! Set first record in the climatological case:   
          IF ((ln_bdy_clim).AND.(kbdy==1)) THEN
            nbdy2 = 1
          ELSEIF ((ln_bdy_clim).AND.(kbdy==iman)) THEN
            nbdy1 = 0
            nbdy2 = imois
          ELSE
            nbdy2 = nbdy1
          END IF
 
         ! Open Netcdf files:

          CALL iom_open ( filbdy_data_T, bdynumt )
          CALL iom_open ( filbdy_data_U, bdynumu )
          CALL iom_open ( filbdy_data_V, bdynumv )

         ! Read first record:
          ipj=1
          ipk=jpk
          jgrd=1
          ipi=nblendta(jgrd)

#if ! defined key_barotropic
          !temperature

          jgrd=1
          IF ( nblendta(jgrd) .le. 0 ) THEN 
            idvar = iom_varid( bdynumt,'votemper' )
            nblendta(jgrd) = iom_file(bdynumt)%dimsz(1,idvar)
          ENDIF
          WRITE(numout,*) 'Dim size for votemper is ',nblendta(jgrd)
          ipi=nblendta(jgrd)

          CALL iom_get ( bdynumt, jpdom_unknown,'votemper',pdta3(1:ipi,1:ipj,1:ipk),nbdy2 )

          DO jb=1, nblen(jgrd)
            DO jk=1,jpkm1
              tbdydta(jb,jk,2) =  pdta3(nbmap(jb,jgrd),1,jk)
            END DO
          END DO

          ! salinity

          jgrd=1
          IF ( nblendta(jgrd) .le. 0 ) THEN 
            idvar = iom_varid( bdynumt,'vosaline' )
            nblendta(jgrd) = iom_file(bdynumt)%dimsz(1,idvar)
          ENDIF
          WRITE(numout,*) 'Dim size for vosaline is ',nblendta(jgrd)
          ipi=nblendta(jgrd)

          CALL iom_get ( bdynumt, jpdom_unknown,'vosaline',pdta3(1:ipi,1:ipj,1:ipk),nbdy2 )

          DO jb=1, nblen(jgrd)
            DO jk=1,jpkm1
              sbdydta(jb,jk,2) =  pdta3(nbmap(jb,jgrd),1,jk)
            END DO
          END DO
#endif
          
          ! u-velocity

          jgrd=2
          IF ( nblendta(jgrd) .le. 0 ) THEN 
            idvar = iom_varid( bdynumu,'vozocrtx' )
            nblendta(jgrd) = iom_file(bdynumu)%dimsz(1,idvar)
          ENDIF
          WRITE(numout,*) 'Dim size for vozocrtx is ',nblendta(jgrd)
          ipi=nblendta(jgrd)

          CALL iom_get ( bdynumu, jpdom_unknown,'vozocrtx',pdta3(1:ipi,1:ipj,1:ipk),nbdy2 )

          DO jb=1, nblen(jgrd)
            DO jk=1,jpkm1
              ubdydta(jb,jk,2) =  pdta3(nbmap(jb,jgrd),1,jk)
            END DO
          END DO

          ! v-velocity
          jgrd=3
          IF ( nblendta(jgrd) .le. 0 ) THEN 
            idvar = iom_varid( bdynumv,'vomecrty' )
            nblendta(jgrd) = iom_file(bdynumv)%dimsz(1,idvar)
          ENDIF
          WRITE(numout,*) 'Dim size for vomecrty is ',nblendta(jgrd)
          ipi=nblendta(jgrd)

          CALL iom_get ( bdynumv, jpdom_unknown,'vomecrty',pdta3(1:ipi,1:ipj,1:ipk),nbdy2 )

          DO jb=1, nblen(jgrd)
            DO jk=1,jpkm1
              vbdydta(jb,jk,2) =  pdta3(nbmap(jb,jgrd),1,jk)
            END DO
          END DO

        END IF
 
        ! In the case of constant boundary forcing fill bdy arrays once for all
        IF ((ln_bdy_clim).AND.(kbdy==1)) THEN
#if ! defined key_barotropic
          tbdy  (:,:) = tbdydta  (:,:,2)
          sbdy  (:,:) = sbdydta  (:,:,2)
#endif
          ubdy  (:,:) = ubdydta  (:,:,2)
          vbdy  (:,:) = vbdydta  (:,:,2)

          CALL iom_close( bdynumt )
          CALL iom_close( bdynumu )
          CALL iom_close( bdynumv )
        END IF

      ENDIF ! End if nit000

      ! -------------------- !
      ! 2. At each time step !
      ! -------------------- !

      IF ((nbdy_dta==1).AND.(kbdy>1)) THEN 

      ! 2.1 Read one more record if necessary
      !**************************************

        IF ( (ln_bdy_clim).AND.(imois/=nbdy1) ) THEN ! remember that nbdy1=0 for kt=nit000
         nbdy1 = imois
         nbdy2 = imois+1
         nbdy1 = MOD( nbdy1, iman )
         IF( nbdy1 == 0 ) nbdy1 = iman
         nbdy2 = MOD( nbdy2, iman )
         IF( nbdy2 == 0 ) nbdy2 = iman
         lect=.true.

        ELSEIF ((.NOT.ln_bdy_clim).AND.(itimer >= istep(nbdy2))) THEN
          nbdy1=nbdy2
          nbdy2=nbdy2+1
          lect=.true.
        END IF
         
        IF (lect) THEN

        ! Swap arrays
#if ! defined key_barotropic
          tbdydta(:,:,1) =  tbdydta(:,:,2)
          sbdydta(:,:,1) =  sbdydta(:,:,2)
#endif
          ubdydta(:,:,1) =  ubdydta(:,:,2)
          vbdydta(:,:,1) =  vbdydta(:,:,2)
 
        ! read another set

          ipj=1
          ipk=jpk
          jgrd=1
          ipi=nblendta(jgrd)

#if ! defined key_barotropic
          !temperature
 
          jgrd=1
          ipi=nblendta(jgrd)

          CALL iom_get ( bdynumt, jpdom_unknown,'votemper',pdta3(1:ipi,1:ipj,1:ipk),nbdy2 )

          DO jb=1, nblen(jgrd)
            DO jk=1,jpkm1
              tbdydta(jb,jk,2) =  pdta3(nbmap(jb,jgrd),1,jk)
            END DO
          END DO

          ! salinity

          jgrd=1
          ipi=nblendta(jgrd)

          CALL iom_get ( bdynumt, jpdom_unknown,'vosaline',pdta3(1:ipi,1:ipj,1:ipk),nbdy2 )

          DO jb=1, nblen(jgrd)
            DO jk=1,jpkm1
              sbdydta(jb,jk,2) =  pdta3(nbmap(jb,jgrd),1,jk)
            END DO
          END DO
#endif
          
          ! u-velocity

          jgrd=2
          ipi=nblendta(jgrd)

          CALL iom_get ( bdynumu, jpdom_unknown,'vozocrtx',pdta3(1:ipi,1:ipj,1:ipk),nbdy2 )

          DO jb=1, nblen(jgrd)
            DO jk=1,jpkm1
              ubdydta(jb,jk,2) =  pdta3(nbmap(jb,jgrd),1,jk)
            END DO
          END DO

          ! v-velocity
          jgrd=3
          ipi=nblendta(jgrd)

          CALL iom_get ( bdynumv, jpdom_unknown,'vomecrty',pdta3(1:ipi,1:ipj,1:ipk),nbdy2 )

          DO jb=1, nblen(jgrd)
            DO jk=1,jpkm1
              vbdydta(jb,jk,2) =  pdta3(nbmap(jb,jgrd),1,jk)
            END DO
          END DO

         IF(lwp) WRITE(numout,*) 'bdy_dta : first record file used nbdy1 ',nbdy1
         IF(lwp) WRITE(numout,*) '~~~~~~~~  last  record file used nbdy2 ',nbdy2
         IF (.NOT.ln_bdy_clim) THEN
           IF(lwp) WRITE(numout,*) 'first  record time (s): ', istep(nbdy1)
           IF(lwp) WRITE(numout,*) 'model time (s)        : ', itimer
           IF(lwp) WRITE(numout,*) 'second record time (s): ', istep(nbdy2)
         ENDIF
        END IF ! end lect=.true.


      ! 2.2   Interpolate linearly:
      ! ***************************
    
        IF (ln_bdy_clim) THEN
          zxy = FLOAT( nday ) / FLOAT( nobis(nbdy1) ) + 0.5 - i15
        ELSE          
          zxy = FLOAT(istep(nbdy1)-itimer) / FLOAT(istep(nbdy1)-istep(nbdy2))
        END IF

#if ! defined key_barotropic
          jgrd=1
          DO jb=1, nblen(jgrd)
            DO jk=1, jpkm1
              tbdy(jb,jk) = zxy      * tbdydta(jb,jk,2) + &
                            (1.-zxy) * tbdydta(jb,jk,1)
              sbdy(jb,jk) = zxy      * sbdydta(jb,jk,2) + &
                            (1.-zxy) * sbdydta(jb,jk,1)
            END DO
          END DO
#endif

          jgrd=2
          DO jb=1, nblen(jgrd)
            DO jk=1, jpkm1
              ubdy(jb,jk) = zxy      * ubdydta(jb,jk,2) + &
                            (1.-zxy) * ubdydta(jb,jk,1)   
            END DO
          END DO

          jgrd=3
          DO jb=1, nblen(jgrd)
            DO jk=1, jpkm1
              vbdy(jb,jk) = zxy      * vbdydta(jb,jk,2) + &
                            (1.-zxy) * vbdydta(jb,jk,1)   
            END DO
          END DO

      END IF !end if ((nbdy_dta==1).AND.(kbdy>1))
    
      ! ------------------- !
      ! Last call kt=nitend !
      ! ------------------- !

      ! Closing of the 3 files
      IF( kt == nitend ) THEN
          CALL iom_close( bdynumt )
          CALL iom_close( bdynumu )
          CALL iom_close( bdynumv )
      ENDIF

   END SUBROUTINE bdy_dta

   SUBROUTINE bdy_dta_bt( kt, jit )
      !!---------------------------------------------------------------------------
      !!                      ***  SUBROUTINE bdy_dta_bt  ***
      !!                    
      !! ** Purpose :   Read unstructured boundary data for barotropic variables
      !!
      !! ** Method  :   
      !!
      !! History : OPA 9.0  ! 07-07 (D. Storkey) Original
      !!---------------------------------------------------------------------------
      !! * Arguments
      INTEGER, INTENT( in ) ::   kt          ! ocean time-step index
      INTEGER, INTENT( in ) ::   jit         ! barotropic time step index
                                             ! (for timesplitting option, otherwise zero)

      !! * Local declarations
      LOGICAL ::   lect                      ! flag for reading
      INTEGER ::   jt, jb, jgrd              ! dummy loop indices
      INTEGER ::   idvar                     ! netcdf var ID
      INTEGER ::   iman, i15, imois          ! Time variables for monthly clim forcing
      INTEGER ::   kbdyt, kbdyu, kbdyv
      INTEGER ::   itimer, totime
      INTEGER ::   ipi, ipj, ipk, inum       ! temporary integers (NetCDF read)
      INTEGER ::   iyear0, imonth0, iday0
      INTEGER ::   ihours0, iminutes0, isec0
      INTEGER ::   kyear, kmonth, kday, ksecs
      INTEGER, DIMENSION(jpbtime) ::   istept, istepu, istepv   ! time arrays from data files
      REAL(wp) ::   dayfrac, zxy, offsett
      REAL(wp) ::   offsetu, offsetv
      REAL(wp) ::   dayjul0, zdayjulini
      REAL(wp), DIMENSION(jpbtime)      ::   istepr             ! REAL time array from data files
      REAL(wp), DIMENSION(jpbdta,1)     ::   pdta2              ! temporary array for data fields
      REAL(wp), DIMENSION(jpbdta,1,jpk) ::   pdta3              ! temporary array for data fields
      CHARACTER(LEN=80), DIMENSION(3)   ::   bdyfile
      CHARACTER(LEN=70 )                ::   clunits            ! units attribute of time coordinate

      !!---------------------------------------------------------------------------

      ! -------------------- !
      !    Initialization    !
      ! -------------------- !

      lect   = .false.           ! If true, read a time record

      ! Some time variables for monthly climatological forcing:
      ! *******************************************************
 
      iman  = INT( raamo ) ! Number of months in a year

      i15 = INT( 2*FLOAT( nday ) / ( FLOAT( nobis(nmonth) ) + 0.5 ) )
      ! i15=0 if the current day is in the first half of the month, else i15=1

      imois = nmonth + i15 - 1            ! imois is the first month record
      IF( imois == 0 ) imois = iman

      ! Time variable for non-climatological forcing:
      ! *********************************************

      itimer = ((kt-1)-nit000+1)*rdt      ! current time in seconds for interpolation 
      itimer = itimer + jit*rdtbt         ! in non-climatological case

      ! -------------------------------------!
      ! Update BDY fields with tidal forcing !
      ! -------------------------------------!  

      IF ( lk_bdy_tides ) CALL tide_update( kt, jit ) 
  
      IF ( lk_bdy ) THEN 

      ! -------------------- !
      ! 1.   First call only !
      ! -------------------- !

      IF ( kt == nit000 ) THEN

        istep_bt(:) = 0

        nbdy1_bt=0
        nbdy2_bt=0

      ! 1.1  Get time information from bdy data file
      ! ********************************************

        IF(lwp) WRITE(numout,*)
        IF(lwp) WRITE(numout,*)    'bdy_dta_bt :Initialize unstructured boundary data for barotropic variables.'
        IF(lwp) WRITE(numout,*)    '~~~~~~~' 

        IF (nbdy_dta == 0) THEN
          IF(lwp) WRITE(numout,*)  'Bdy data are taken from initial conditions'

        ELSEIF (nbdy_dta == 1) THEN
          IF(lwp) WRITE(numout,*)  'Bdy data are read in netcdf files'

          dayfrac = adatrj-float(itimer)/86400. ! day fraction at time step kt-1
          dayfrac = dayfrac - INT(dayfrac)      !
          totime = (nitend-nit000+1)*rdt ! Total time of the run to verify that all the
                                           ! necessary time dumps in file are included

          bdyfile(1) = filbdy_data_bt_T
          bdyfile(2) = filbdy_data_bt_U
          bdyfile(3) = filbdy_data_bt_V

          DO jgrd = 1,3

            CALL iom_open( bdyfile(jgrd), inum )
            CALL iom_gettime( inum, istepr, kntime=kbdy, cdunits=clunits ) 
            CALL iom_close( inum )

            ! Calculate time offset 
            READ(clunits,7000) iyear0, imonth0, iday0, ihours0, iminutes0, isec0
            ! Convert time origin in file to julian days 
            isec0 = isec0 + ihours0*60.*60. + iminutes0*60.
            CALL ymds2ju(iyear0, imonth0, iday0, real(isec0), dayjul0)
            ! Compute model initialization time 
            kyear  = ndastp / 10000
            kmonth = ( ndastp - kyear * 10000 ) / 100
            kday   = ndastp - kyear * 10000 - kmonth * 100
            ksecs  = dayfrac * 86400
            CALL ymds2ju(kyear, kmonth, kday, real(ksecs) , zdayjulini)
            ! offset from initialization date:
            offset = (dayjul0-zdayjulini)*86400
            !

7000 FORMAT('seconds since ', I4.4,'-',I2.2,'-',I2.2,' ',I2.2,':',I2.2,':',I2.2)

            !! TO BE DONE... Check consistency between calendar from file 
            !! (available optionally from iom_gettime) and calendar in model 
            !! when calendar in model available outside of IOIPSL.

            IF(lwp) WRITE(numout,*) 'kdby (number of times): ',kbdy_bt
            IF(lwp) WRITE(numout,*) 'offset: ',offset
            IF(lwp) WRITE(numout,*) 'totime: ',totime
            IF(lwp) WRITE(numout,*) 'istepr: ',istepr

            ! Check that there are not too many times in the file. 
            IF (kbdy_bt > jpbtime) THEN
              IF(lwp) WRITE(numout,*)
              IF(lwp) WRITE(numout,*) ' E R R O R : Number of time dumps in files exceed jpbtime parameter'
              IF(lwp) WRITE(numout,*) ' ==========  jpbtime= ',jpbtime,' kbdy_bt= ', kbdy_bt             
              IF(lwp) WRITE(numout,*) ' Check file: ', bdyfile(jgrd)
              nstop = nstop + 1
            ENDIF

            ! Check that time array increases:
        
            jt=1
            DO WHILE ((istepr(jt+1).GT.istepr(jt)).AND.(jt.LE.(kbdy_bt-1)))
              jt=jt+1
            END DO

            IF ((jt.NE.kbdy_bt).AND.(kbdy_bt.GT.1)) THEN
              IF(lwp) WRITE(numout,*)
              IF(lwp) WRITE(numout,*) ' E R R O R : Time array in unstructured boundary data files'
              IF(lwp) WRITE(numout,*) ' ==========  does not continuously increase. Check file: '         
              IF(lwp) WRITE(numout,*) bdyfile(jgrd)
              IF(lwp) WRITE(numout,*)
              nstop = nstop + 1
            ENDIF

            ! Check that times in file span model run time:
 
            IF (istepr(1)+offset > 0) THEN
              IF(lwp) WRITE(numout,*)
              IF(lwp) WRITE(numout,*) ' E R R O R : First time dump in bdy file is after model initial time'
              IF(lwp) WRITE(numout,*) ' ==========  Check file: ',bdyfile(jgrd)
              IF(lwp) WRITE(numout,*)
              nstop = nstop + 1
            END IF

            IF (istepr(kbdy_bt)+offset < totime) THEN
              IF(lwp) WRITE(numout,*)
              IF(lwp) WRITE(numout,*) ' E R R O R : Last time dump in bdy file is before model final time'
              IF(lwp) WRITE(numout,*) ' ==========  Check file: ',bdyfile(jgrd) 
              IF(lwp) WRITE(numout,*)
              nstop = nstop + 1
            END IF

            IF ( jgrd .EQ. 1) THEN
              kbdyt = kbdy_bt
              offsett = offset
              istept(:) = INT( istepr(:) + offset )
            ELSE IF (jgrd .EQ. 2) THEN
              kbdyu = kbdy_bt
              offsetu = offset
              istepu(:) = INT( istepr(:) + offset )
            ELSE IF (jgrd .EQ. 3) THEN
              kbdyv = kbdy_bt
              offsetv = offset
              istepv(:) = INT( istepr(:) + offset )
            ENDIF

          ENDDO

      ! Verify time consistency between files  

          IF (kbdyu.NE.kbdyt .OR. kbdyv.NE.kbdyt) THEN
            IF(lwp) WRITE(numout,*) 'Bdy data files must have the same number of time dumps'
            IF(lwp) WRITE(numout,*) 'Multiple time frequencies not implemented yet'
            nstop = nstop + 1
          ENDIF
          kbdy_bt = kbdyt

          IF (offsetu.NE.offsett .OR. offsetv.NE.offsett) THEN
            IF(lwp) WRITE(numout,*) 'Bdy data files must have the same time origin'
            IF(lwp) WRITE(numout,*) 'Multiple time frequencies not implemented yet'
            nstop = nstop + 1
          ENDIF
          offset = offsett

      !! Check that times are the same in the three files... HERE.
          istep_bt(:) = istept(:)

      ! Check number of time dumps:              
          IF ((kbdy_bt.EQ.1).AND.(.NOT.ln_bdy_clim)) THEN
            IF(lwp) WRITE(numout,*)
            IF(lwp) WRITE(numout,*) ' E R R O R : There is only one time dump in data files'
            IF(lwp) WRITE(numout,*) ' ==========  Choose ln_bdy_clim=.true. in namelist for constant bdy forcing.'            
            IF(lwp) WRITE(numout,*)
            nstop = nstop + 1
          ENDIF

          IF (ln_bdy_clim) THEN
            IF ((kbdy_bt.NE.1).AND.(kbdy_bt.NE.12)) THEN
              IF(lwp) WRITE(numout,*)
              IF(lwp) WRITE(numout,*) ' E R R O R : For climatological boundary forcing (ln_bdy_clim=.true.),'
              IF(lwp) WRITE(numout,*) ' ==========  bdy data files must contain 1 or 12 time dumps.'         
              IF(lwp) WRITE(numout,*)
              nstop = nstop + 1
            ELSEIF (kbdy_bt.EQ.1 ) THEN
              IF(lwp) WRITE(numout,*)
              IF(lwp) WRITE(numout,*) 'We assume constant boundary forcing from bdy data files'
              IF(lwp) WRITE(numout,*)             
            ELSEIF (kbdy_bt.EQ.12) THEN
              IF(lwp) WRITE(numout,*)
              IF(lwp) WRITE(numout,*) 'We assume monthly (and cyclic) boundary forcing from bdy data files'
              IF(lwp) WRITE(numout,*) 
            ENDIF
          ENDIF

      ! Find index of first record to read (before first model time). 

          jt=1
          DO WHILE ( ((istep_bt(jt+1)) <= 0 ).AND.(jt.LE.(kbdy_bt-1)))
            jt=jt+1
          END DO
          nbdy1_bt = jt

          WRITE(numout,*) 'Time offset is ',offset
          WRITE(numout,*) 'First record to read is ',nbdy1_bt

        ENDIF ! endif (nbdy_dta == 1)

      ! 1.2  Read first record in file if necessary (ie if nbdy_dta == 1)
      ! *****************************************************************

        IF ( nbdy_dta == 0) THEN
          ! boundary data arrays are filled with initial conditions
          jgrd = 2            ! U-points data 
          DO jb = 1, nblen(jgrd)              
            ubtbdy(jb) = un(nbi(jb,jgrd), nbj(jb,jgrd), 1)
          END DO

          jgrd = 3            ! V-points data 
          DO jb = 1, nblen(jgrd)              
            vbtbdy(jb) = vn(nbi(jb,jgrd), nbj(jb,jgrd), 1)
          END DO

          jgrd = 1            ! T-points data 
          DO jb = 1, nblen(jgrd)              
            sshbdy(jb) = sshn(nbi(jb,jgrd), nbj(jb,jgrd))
          END DO

        ELSEIF (nbdy_dta == 1) THEN
 
        ! Set first record in the climatological case:   
          IF ((ln_bdy_clim).AND.(kbdy_bt==1)) THEN
            nbdy2_bt = 1
          ELSEIF ((ln_bdy_clim).AND.(kbdy_bt==iman)) THEN
            nbdy1_bt = 0
            nbdy2_bt = imois
          ELSE
            nbdy2_bt = nbdy1_bt
          END IF
 
         ! Open Netcdf files:

          CALL iom_open ( filbdy_data_bt_T, bdynumt_bt )
          CALL iom_open ( filbdy_data_bt_U, bdynumu_bt )
          CALL iom_open ( filbdy_data_bt_V, bdynumv_bt )

         ! Read first record:
          ipj=1
          jgrd=1
          ipi=nblendta(jgrd)

          ! ssh
          jgrd=1
          IF ( nblendta(jgrd) .le. 0 ) THEN 
            idvar = iom_varid( bdynumt_bt,'sossheig' )
            nblendta(jgrd) = iom_file(bdynumt_bt)%dimsz(1,idvar)
          ENDIF
          WRITE(numout,*) 'Dim size for sossheig is ',nblendta(jgrd)
          ipi=nblendta(jgrd)

          CALL iom_get ( bdynumt_bt, jpdom_unknown,'sossheig',pdta2(1:ipi,1:ipj),nbdy2_bt )

          DO jb=1, nblen(jgrd)
            sshbdydta(jb,2) =  pdta2(nbmap(jb,jgrd),1)
          END DO
 
          ! u-velocity
          jgrd=2
          IF ( nblendta(jgrd) .le. 0 ) THEN 
            idvar = iom_varid( bdynumu_bt,'vobtcrtx' )
            nblendta(jgrd) = iom_file(bdynumu_bt)%dimsz(1,idvar)
          ENDIF
          WRITE(numout,*) 'Dim size for vobtcrtx is ',nblendta(jgrd)
          ipi=nblendta(jgrd)

          CALL iom_get ( bdynumu_bt, jpdom_unknown,'vobtcrtx',pdta2(1:ipi,1:ipj),nbdy2_bt )

          DO jb=1, nblen(jgrd)
            ubtbdydta(jb,2) =  pdta2(nbmap(jb,jgrd),1)
          END DO

          ! v-velocity
          jgrd=3
          IF ( nblendta(jgrd) .le. 0 ) THEN 
            idvar = iom_varid( bdynumv_bt,'vobtcrty' )
            nblendta(jgrd) = iom_file(bdynumv_bt)%dimsz(1,idvar)
          ENDIF
          WRITE(numout,*) 'Dim size for vobtcrty is ',nblendta(jgrd)
          ipi=nblendta(jgrd)

          CALL iom_get ( bdynumv_bt, jpdom_unknown,'vobtcrty',pdta2(1:ipi,1:ipj),nbdy2_bt )

          DO jb=1, nblen(jgrd)
            vbtbdydta(jb,2) =  pdta2(nbmap(jb,jgrd),1)
          END DO

        END IF
 
        ! In the case of constant boundary forcing fill bdy arrays once for all
        IF ((ln_bdy_clim).AND.(kbdy_bt==1)) THEN

          ubtbdy  (:) = ubtbdydta  (:,2)
          vbtbdy  (:) = vbtbdydta  (:,2)
          sshbdy  (:) = sshbdydta  (:,2)

          CALL iom_close( bdynumt_bt )
          CALL iom_close( bdynumu_bt )
          CALL iom_close( bdynumv_bt )

        END IF

      ENDIF ! End if nit000

      ! -------------------- !
      ! 2. At each time step !
      ! -------------------- !

      IF ((nbdy_dta==1).AND.(kbdy_bt>1)) THEN 

      ! 2.1 Read one more record if necessary
      !**************************************

        IF ( (ln_bdy_clim).AND.(imois/=nbdy1_bt) ) THEN ! remember that nbdy1_bt=0 for kt=nit000
         nbdy1_bt = imois
         nbdy2_bt = imois+1
         nbdy1_bt = MOD( nbdy1_bt, iman )
         IF( nbdy1_bt == 0 ) nbdy1_bt = iman
         nbdy2_bt = MOD( nbdy2_bt, iman )
         IF( nbdy2_bt == 0 ) nbdy2_bt = iman
         lect=.true.

        ELSEIF ((.NOT.ln_bdy_clim).AND.(itimer >= istep_bt(nbdy2_bt))) THEN
          nbdy1_bt=nbdy2_bt
          nbdy2_bt=nbdy2_bt+1
          lect=.true.
        END IF
         
        IF (lect) THEN

        ! Swap arrays
          sshbdydta(:,1) =  sshbdydta(:,2)
          ubtbdydta(:,1) =  ubtbdydta(:,2)
          vbtbdydta(:,1) =  vbtbdydta(:,2)
 
        ! read another set

          ipj=1
          ipk=jpk
          jgrd=1
          ipi=nblendta(jgrd)

          
          ! ssh
          jgrd=1
          ipi=nblendta(jgrd)

          CALL iom_get ( bdynumt_bt, jpdom_unknown,'sossheig',pdta2(1:ipi,1:ipj),nbdy2_bt )

          DO jb=1, nblen(jgrd)
            sshbdydta(jb,2) =  pdta2(nbmap(jb,jgrd),1)
          END DO

          ! u-velocity
          jgrd=2
          ipi=nblendta(jgrd)

          CALL iom_get ( bdynumu_bt, jpdom_unknown,'vobtcrtx',pdta2(1:ipi,1:ipj),nbdy2_bt )

          DO jb=1, nblen(jgrd)
            ubtbdydta(jb,2) =  pdta3(nbmap(jb,jgrd),1)
          END DO

          ! v-velocity
          jgrd=3
          ipi=nblendta(jgrd)

          CALL iom_get ( bdynumv_bt, jpdom_unknown,'vobtcrty',pdta2(1:ipi,1:ipj),nbdy2_bt )

          DO jb=1, nblen(jgrd)
            vbtbdydta(jb,2) =  pdta3(nbmap(jb,jgrd),1)
          END DO


         IF(lwp) WRITE(numout,*) 'bdy_dta : first record file used nbdy1_bt ',nbdy1_bt
         IF(lwp) WRITE(numout,*) '~~~~~~~~  last  record file used nbdy2_bt ',nbdy2_bt
         IF (.NOT.ln_bdy_clim) THEN
           IF(lwp) WRITE(numout,*) 'first  record time (s): ', istep_bt(nbdy1_bt)
           IF(lwp) WRITE(numout,*) 'model time (s)        : ', itimer
           IF(lwp) WRITE(numout,*) 'second record time (s): ', istep_bt(nbdy2_bt)
         ENDIF
        END IF ! end lect=.true.


      ! 2.2   Interpolate linearly:
      ! ***************************
    
        IF (ln_bdy_clim) THEN
          zxy = FLOAT( nday ) / FLOAT( nobis(nbdy1_bt) ) + 0.5 - i15
        ELSE          
          zxy = FLOAT(istep_bt(nbdy1_bt)-itimer) / FLOAT(istep_bt(nbdy1_bt)-istep_bt(nbdy2_bt))
        END IF

          jgrd=1
          DO jb=1, nblen(jgrd)
            sshbdy(jb) = zxy      * sshbdydta(jb,2) + &
                       (1.-zxy) * sshbdydta(jb,1)   
          END DO

          jgrd=2
          DO jb=1, nblen(jgrd)
            ubtbdy(jb) = zxy      * ubtbdydta(jb,2) + &
                         (1.-zxy) * ubtbdydta(jb,1)   
          END DO

          jgrd=3
          DO jb=1, nblen(jgrd)
            vbtbdy(jb) = zxy      * vbtbdydta(jb,2) + &
                         (1.-zxy) * vbtbdydta(jb,1)   
          END DO


      END IF !end if ((nbdy_dta==1).AND.(kbdy_bt>1))
    
      ! ------------------- !
      ! Last call kt=nitend !
      ! ------------------- !

      ! Closing of the 3 files
      IF( kt == nitend ) THEN
          CALL iom_close( bdynumt_bt )
          CALL iom_close( bdynumu_bt )
          CALL iom_close( bdynumv_bt )
      ENDIF

      ENDIF ! lk_bdy

      END SUBROUTINE bdy_dta_bt


#else
   !!------------------------------------------------------------------------------
   !!   default option:           Dummy module NO Unstruct Open Boundary Conditions
   !!------------------------------------------------------------------------------
CONTAINS
   SUBROUTINE bdy_dta( kt )             ! Dummy routine
      INTEGER, INTENT (in) :: kt
      WRITE(*,*) 'bdy_dta: You should not have seen this print! error?', kt
   END SUBROUTINE bdy_dta

   SUBROUTINE bdy_dta_bt( kt, jit )             ! Dummy routine
      INTEGER, INTENT (in) :: kt, jit
      WRITE(*,*) 'bdy_dta: You should not have seen this print! error?', kt, jit
   END SUBROUTINE bdy_dta_bt
#endif

   !!==============================================================================
END MODULE bdydta
